#!/usr/bin/env python

"""
Twisted classes.
"""

import Generic

class Transaction(Generic.Transaction):

    """
    Twisted transaction interface.
    """

    def __init__(self, trans):

        "Initialise the transaction using the Twisted transaction 'trans'."

        self.trans = trans

    # Request-related methods.

    def get_request_stream(self):

        """
        A framework-specific method which returns the request stream for
        the transaction.
        """

        return self.trans.content

    def get_request_method(self):

        """
        A framework-specific method which gets the request method.
        """

        return self.trans.method

    def get_headers(self):

        """
        A framework-specific method which returns the request headers.
        NOTE: Experimental, since framework support varies somewhat.
        """

        # NOTE: Accessing attribute of transaction object.

        return self.trans.received_headers

    def get_content_type(self):

        """
        A framework-specific method which gets the content type specified on the
        request, along with the charset employed.
        """

        return self.parse_content_type(self.trans.getHeader("Content-Type"))

    def get_content_charsets(self):

        """
        Returns the character set preferences.
        """

        return self.parse_content_preferences(self.trans.getHeader("Accept-Language"))

    def get_content_languages(self):

        """
        A framework-specific method which extracts language information from
        the transaction.
        """

        return self.parse_content_preferences(self.trans.getHeader("Accept-Charset"))

    def get_path(self):

        """
        A framework-specific method which gets the entire path from the request.
        """

        # NOTE: The path and "path info" are mostly equivalent for Twisted.

        return self.get_path_info()

    def get_path_info(self):

        """
        A framework-specific method which gets the "path info" (the part of the
        URL after the resource name handling the current request) from the
        request.
        """

        return "/%s" % "/".join(self.trans.postpath)

    # Higher level request-related methods.

    def get_fields(self):

        """
        A framework-specific method which extracts the form fields from the
        transaction.
        """

        # NOTE: Discard multiple field values.

        return dict([(key, value[0]) for (key, value) in self.trans.args.items()])

    def get_agent_information(self):

        """
        A framework-specific method which extracts agent information from
        the transaction.
        """

        return None

    # Response-related methods.

    def get_response_stream(self):

        """
        A framework-specific method which returns the response stream for
        the transaction.
        """

        return self.trans

    def set_content_type(self, content_type):

        """
        A framework-specific method which sets the 'content_type' for the
        response.
        """

        self.trans.setHeader("Content-Type", self.format_content_type(content_type))

# vim: tabstop=4 expandtab shiftwidth=4
